-- get the addon namespace
local addon, ns = ...
-- get the config values
local cfg = ns.cfg
-- holder for some lib functions
local lib = _G["HEDD_lib"] or CreateFrame("Frame","HEDD_lib")
ns.lib = lib

lib.classes = {}
lib.classevents = {}

lib.inrange = function(spell)
	if cfg.spells[spell] then 
		if IsSpellInRange(cfg.spells[spell].name,"target")==1 then return true end
	else
		if IsSpellInRange(spell,"target")==1 then return true end
	end
	return nil
end

lib.GroundUpdate = function()
	tmp=cfg.onGround
	if IsFalling() or IsFlying() then
		cfg.onGround=nil
	else
		cfg.onGround=true
	end
	
	if tmp~=cfg.onGround then
		cfg.Update=true
		return true
	end
	return nil
end

lib.UpdateShape = function()
	tmp=cfg.shape
	cfg.shapeid=GetShapeshiftFormID()
	if cfg.shapeid and cfg.shapes[cfg.shapeid] then
		cfg.shape=cfg.shapes[cfg.shapeid]
	else 
		cfg.shape="human"
	end
	if tmp~=cfg.shape then
		cfg.Update=true
		return true
	end
	return nil
end

lib.UpdateSet = function()
	cfg.setitems={}
	for itemSET, _ in pairs(cfg.set) do
		cfg.setitems[itemSET]=0
		for _, itemID in pairs(cfg.set[itemSET]) do
			for i=0,19 do
				if cfg.equip[i] and cfg.equip[i]==itemID then
					cfg.setitems[itemSET]=cfg.setitems[itemSET]+1
				end
			end
		end
	end
end

lib.SetBonus=function(set)
	if cfg.setitems[set] then
		if cfg.setitems[set]>=4 then
			return 2
		elseif cfg.setitems[set]>=2 then
			return 1
		end
	end
	return 0
end

lib.UpdateEquip=function(slot)
	cfg.equip[slot]=GetInventoryItemID("player", slot)
end

lib.UpdateEquipAll=function()
	for i=0,19 do
		lib.UpdateEquip(i)
	end
	lib.UpdateSet()
end

lib.SwitchMode = function()
	if cfg.mode~="dps" then
		for _,cspell in ipairs(cfg.spells_single) do
			if cfg.spells[cspell] and (cfg.lastspell==cfg.spells[cspell].id or (cfg.casting and cfg.casting==cfg.spells[cspell].name)) then
				cfg.mode = "dps"
				cfg.plist=cfg.plistdps
				cfg.Update=true
				return
			end
		end
	end
	
	if cfg.mode~="aoe" then
		for _,cspell in ipairs(cfg.spells_aoe) do
			if cfg.spells[cspell] and (cfg.lastspell==cfg.spells[cspell].id or (cfg.casting and cfg.casting==cfg.spells[cspell].name)) then
				cfg.mode = "aoe"
				cfg.plist=cfg.plistaoe
				cfg.Update=true
				return
			end
		end
	end
	
	if cfg.mode~="range" then
		for _,cspell in ipairs(cfg.spells_range) do
			if cfg.spells[cspell] and (cfg.lastspell==cfg.spells[cspell].name or (cfg.casting and cfg.casting==cfg.spells[cspell].name)) then
				cfg.mode = "range"
				cfg.plist=cfg.plistrange
				cfg.Update=true
				return
			end
		end
	end
end

lib.Power = function()
	cfg.realpower=cfg.power
	if cfg.casting and cfg.sp_conv[cfg.casting] and cfg.castEnd>cfg.now then --and ((cfg.showchannel and cfg.ischanneling) or (not cfg.ischanneling)) 
		cfg.realpower=cfg.power+(cfg.castEnd-cfg.now)*cfg.pactiveRegen-cfg.spells[cfg.sp_conv[cfg.casting]].powerCost
	end
	if cfg.realpower>cfg.powermax then cfg.realpower=cfg.powermax end 
	return cfg.realpower
end


lib.instance=function()
	cfg.instance=select(2,IsInInstance())
	--[[
	arena - Player versus player arena
	none - Not inside an instance
	party - 5-man instance
	pvp - Player versus player battleground
	raid - Raid instance
	]]
	return cfg.instance
end

lib.ininstance=function()
	if cfg.instance=="raid" or cfg.instance=="party" then
		return true 
	else
		return nil
	end
end

lib.raidboss = function()
	if cfg.target=="worldboss" and cfg.instance=="raid" then return true end
	return nil
end

lib.UpdateTarget = function()
	cfg.target=UnitClassification("target")
	if UnitLevel("target")==-1 then cfg.target="worldboss" end
	cfg.targetGUID=UnitGUID("target")
	--[[
	elite - Elite
	normal - Normal
	minus - Small
	rare - Rare
	rareelite - Rare-Elite
	worldboss - World Boss
	]]
--	print(cfg.target.." "..cfg.instance)
	return cfg.target
end

lib.hardtarget = function()
	if cfg.target=="normal" or cfg.target=="minus" then return nil end
	return true
end

lib.bosstarget = function()
	if cfg.target=="worldboss" then
		return true
	end
	return nil
end


lib.UpdatePower = function(powerType)
	if not powerType and cfg.powertype then return end
	if cfg.powertype and cfg.powertype~=powerType then
		cfg.Update=true
		return
	end
	ppower=cfg.power
	cfg.power=UnitPower("player",powerType)
	ppowermax=cfg.powermax
	cfg.powermax=UnitPowerMax("player",powerType)
	ppactiveRegen=cfg.pactiveRegen
	_,cfg.pactiveRegen=GetPowerRegen()
	
	cfg.realpower=cfg.power
	if cfg.casting and cfg.sp_conv[cfg.casting] and cfg.castEnd>cfg.now then --and ((cfg.showchannel and cfg.ischanneling) or (not cfg.ischanneling)) 
		cfg.realpower=cfg.power+(cfg.castEnd-cfg.now)*cfg.pactiveRegen-cfg.spells[cfg.sp_conv[cfg.casting]].powerCost
	end
	if cfg.realpower>cfg.powermax then cfg.realpower=cfg.powermax end 
	
	if cfg.power~=ppower or cfg.powermax~=ppowermax or cfg.pactiveRegen~=ppactiveRegen then
		cfg.Update=true
	end
end

lib.UpdateHealth = function()
	phealth=cfg.health
	phealthmax=cfg.healthmax
	cfg.healthmax=UnitHealthMax("target")
	cfg.health=(cfg.healthmax==0) and 100 or hedlib.round(UnitHealth("target")*100/cfg.healthmax,0)

	if cfg.health~=phealth or  cfg.healthmax~=phealthmax then
		cfg.Update=true
	end
end

lib.GetSmallest = function(timers)
	smallest=99999
	for _,timer in ipairs(timers) do
		if smallest>timer then smallest=timer end
	end
	return smallest
end

lib.UpdatePlayerHealth = function()
	playerhealth=cfg.phealth
	playerhealthmax=cfg.phealthmax
	playerhealthperc=cfg.phealthperc
	cfg.phealthmax=UnitHealthMax("player")
	cfg.phealth=UnitHealth("player")
	cfg.phealthperc=(cfg.phealthmax==0) and 100 or hedlib.round(cfg.phealth*100/cfg.phealthmax,0)
	if cfg.phealth~=playerhealth or cfg.phealthmax~=playerhealthmax or cfg.phealthperc~=playerhealthperc then
		cfg.Update=true
	end
end

lib.UpdateHaste = function()
	cfg.haste=GetCombatRatingBonus(CR_HASTE_MELEE)
end

lib.MaxEnergy = function()
	return (cfg.powermax-cfg.realpower)/cfg.pactiveRegen
end

--[[lib.TimeEnergy = function(t)
	return (cfg.realpower+cfg.pactiveRegen*t)
end]]

lib.Time2Energy = function(energy)
	local t=0
	if cfg.realpower<energy then
		t=((energy-cfg.realpower)/cfg.pactiveRegen)
	end
	--print(energy.." "..t)
	return t
end

lib.TimeEnergyGain = function(t)
	return (cfg.pactiveRegen*t)
end

lib.ComboUpdate = function()
	pcombo = cfg.combo
	cfg.combo = GetComboPoints('player', 'target')
	if pcombo~=cfg.combo then
		return true
	end
	return nil
end

cfg.holy = UnitPower("player" , SPELL_POWER_HOLY_POWER)
cfg.holymax = UnitPowerMax("player" , SPELL_POWER_HOLY_POWER)

lib.HolyPowerUpdate = function()
	ppower = cfg.holy
	cfg.holy = UnitPower("player" , SPELL_POWER_HOLY_POWER)
	ppowermax = cfg.chimax
	cfg.holymax=UnitPowerMax("player",SPELL_POWER_HOLY_POWER)
	if ppower~=cfg.holy or ppowermax~=cfg.holymax then
		cfg.Update=true
	end
end

cfg.chi=UnitPower("player" , SPELL_POWER_CHI)
cfg.chimax=UnitPowerMax("player",SPELL_POWER_CHI)
lib.ChiPowerUpdate = function()
	ppower = cfg.chi
	cfg.chi = UnitPower("player" , SPELL_POWER_CHI)
	ppowermax = cfg.chimax
	cfg.chimax=UnitPowerMax("player",SPELL_POWER_CHI)
	if ppower~=cfg.chi or ppowermax~=cfg.chimax then
		cfg.Update=true
	end
end

cfg.orbs=0
lib.OrbsUpdate = function()
	ppower = cfg.orbs
	cfg.orbs = UnitPower("player" , 13)
	if ppower~=cfg.orbs then
		return true
	end
	return nil
end
